@@ -29,7 +29,7 @@ def neo4j_credentials(environment):
         "port": os.getenv("BOT_NEO4J_PORT") or 7687,
         "user": os.getenv("BOT_NEO4J_USER") or 'neo4j',
         "db_name": os.getenv("BOT_NEO4J_NAME") or 'neo4j',
-        "passwd": os.getenv("BOT_NEO4J_PASSWORD") or 'neo4j'
+        "passwd": os.getenv("BOT_NEO4J_PASSWORD") or 'alvarole3552'
 
     }
 
