@@ -13,6 +13,9 @@ from sklearn.preprocessing import LabelEncoder
 from sklearn.metrics import confusion_matrix
 import matplotlib.pyplot as plt
 import seaborn as sns
+import json 
+from pathlib import Path
+
 from nkia.ml.cnn_nlp_model import CNN
 
 def get_dataframe_from_mongo(mongo_conn):
@@ -96,6 +99,7 @@ def encode_class_value(class_value, encoder_path):
 def evaluate_model(cnn, X_test, y_test):
     results = cnn.evaluate(X_test, y_test, batch_size=64)
     print('Loss and Accuracy: ', results)
+    return results[1]
 
 def show_confusion_matrix(y_test, predicted):
     fig, ax = plt.subplots(figsize=(10,10))
@@ -114,9 +118,31 @@ def create_model_instance(y, tokenizer):
     sys.stdout.flush()
     return cnn
 
+def read_file(file_name, content=''):
+    if content:
+        with open(os.path.join(Path(os.path.dirname(__file__)), file_name), 'w') as _file:
+            _file.write(str(content))
+            _file.close()
+    else:
+        with open(os.path.join(Path(os.path.dirname(__file__)), file_name)) as _file:
+            return _file.read()
+
+def get_new_model_version(current_model_version):
+    new_model_version = current_model_version.split('.')
+    new_model_version.insert(1, str(int(new_model_version.pop(1)) + 1))
+    return '.'.join(new_model_version)
+
 def create_saved_model_folder(model_name):
-    if 'saved_model' not in os.listdir('./src/nkia/ml/'):
-        os.makedirs('./src/nkia/ml/saved_model/' + model_name + '/v1')
-    elif model_name not in os.listdir('./src/nkia/ml/saved_model'):
-        os.makedirs('./src/nkia/ml/saved_model/' + model_name + '/v1')
-    
\ No newline at end of file
+    last_model_path = ''
+    current_model_version = json.loads(read_file(os.getcwd() + '/config.txt'))['version']
+    new_model_version = get_new_model_version(current_model_version)
+
+    model_path = os.path.join('./src/nkia/ml/saved_model/' + model_name + '/')
+    os.makedirs(model_path + new_model_version)
+
+    if current_model_version in os.listdir(model_path):
+        last_model_path = model_path + current_model_version        
+    
+    model_path = model_path + new_model_version
+
+    return model_path, last_model_path
